@@ -7,12 +7,16 @@ from unittest import TestCase, SkipTest
 from unittest.mock import patch
 
 import pytest
-from testpath import assert_isfile, assert_isdir, assert_islink, MockCommand
+from testpath import (
+    assert_isfile, assert_isdir, assert_islink, assert_not_path_exists, MockCommand
+)
 
 from flit import install
 from flit.install import Installer, _requires_dist_to_pip_requirement, DependencyError
+import flit_core.tests
 
 samples_dir = pathlib.Path(__file__).parent / 'samples'
+core_samples_dir = pathlib.Path(flit_core.tests.__file__).parent / 'samples'
 
 class InstallTests(TestCase):
     def setUp(self):
@@ -50,6 +54,17 @@ class InstallTests(TestCase):
             samples_dir / 'module1_toml', 'module1', '0.1', expected_editable=False
         )
 
+    def test_install_module_pep621(self):
+        Installer.from_ini_path(
+            core_samples_dir / 'pep621_nodynamic' / 'pyproject.toml',
+        ).install_directly()
+        assert_isfile(self.tmpdir / 'site-packages' / 'module1.py')
+        assert_isdir(self.tmpdir / 'site-packages' / 'module1-0.3.dist-info')
+        self._assert_direct_url(
+            core_samples_dir / 'pep621_nodynamic', 'module1', '0.3',
+            expected_editable=False
+        )
+
     def test_install_package(self):
         oldcwd = os.getcwd()
         os.chdir(str(samples_dir / 'package1'))
@@ -66,6 +81,54 @@ class InstallTests(TestCase):
             samples_dir / 'package1', 'package1', '0.1', expected_editable=False
         )
 
+    def test_install_ns_package_native(self):
+        Installer.from_ini_path(samples_dir / 'ns1-pkg' / 'pyproject.toml').install_directly()
+        assert_isdir(self.tmpdir / 'site-packages' / 'ns1')
+        assert_isfile(self.tmpdir / 'site-packages' / 'ns1' / 'pkg' / '__init__.py')
+        assert_not_path_exists(self.tmpdir / 'site-packages' / 'ns1' / '__init__.py')
+        assert_isdir(self.tmpdir / 'site-packages' / 'ns1_pkg-0.1.dist-info')
+
+    def test_install_ns_package_module_native(self):
+        Installer.from_ini_path(samples_dir / 'ns1-pkg-mod' / 'pyproject.toml').install_directly()
+        assert_isfile(self.tmpdir / 'site-packages' / 'ns1' / 'module.py')
+        assert_not_path_exists(self.tmpdir / 'site-packages' / 'ns1' / '__init__.py')
+
+    def test_install_ns_package_native_symlink(self):
+        if os.name == 'nt':
+            raise SkipTest('symlink')
+        Installer.from_ini_path(
+            samples_dir / 'ns1-pkg' / 'pyproject.toml', symlink=True
+        ).install_directly()
+        Installer.from_ini_path(
+            samples_dir / 'ns1-pkg2' / 'pyproject.toml', symlink=True
+        ).install_directly()
+        Installer.from_ini_path(
+            samples_dir / 'ns1-pkg-mod' / 'pyproject.toml', symlink=True
+        ).install_directly()
+        assert_isdir(self.tmpdir / 'site-packages' / 'ns1')
+        assert_isdir(self.tmpdir / 'site-packages' / 'ns1' / 'pkg')
+        assert_islink(self.tmpdir / 'site-packages' / 'ns1' / 'pkg',
+                      to=str(samples_dir / 'ns1-pkg' / 'ns1' / 'pkg'))
+        assert_isdir(self.tmpdir / 'site-packages' / 'ns1_pkg-0.1.dist-info')
+
+        assert_isdir(self.tmpdir / 'site-packages' / 'ns1' / 'pkg2')
+        assert_islink(self.tmpdir / 'site-packages' / 'ns1' / 'pkg2',
+                      to=str(samples_dir / 'ns1-pkg2' / 'ns1' / 'pkg2'))
+        assert_isdir(self.tmpdir / 'site-packages' / 'ns1_pkg2-0.1.dist-info')
+
+        assert_islink(self.tmpdir / 'site-packages' / 'ns1' / 'module.py',
+                      to=samples_dir / 'ns1-pkg-mod' / 'ns1' / 'module.py')
+        assert_isdir(self.tmpdir / 'site-packages' / 'ns1_module-0.1.dist-info')
+
+    def test_install_ns_package_pth_file(self):
+        Installer.from_ini_path(
+            samples_dir / 'ns1-pkg' / 'pyproject.toml', pth=True
+        ).install_directly()
+
+        pth_file = self.tmpdir / 'site-packages' / 'ns1.pkg.pth'
+        assert_isfile(pth_file)
+        assert pth_file.read_text('utf-8').strip() == str(samples_dir / 'ns1-pkg')
+
     def test_symlink_package(self):
         if os.name == 'nt':
             raise SkipTest("symlink")
@@ -79,6 +142,20 @@ class InstallTests(TestCase):
             samples_dir / 'package1', 'package1', '0.1', expected_editable=True
         )
 
+    def test_symlink_module_pep621(self):
+        if os.name == 'nt':
+            raise SkipTest("symlink")
+        Installer.from_ini_path(
+            core_samples_dir / 'pep621_nodynamic' / 'pyproject.toml', symlink=True
+        ).install_directly()
+        assert_islink(self.tmpdir / 'site-packages' / 'module1.py',
+                      to=core_samples_dir / 'pep621_nodynamic' / 'module1.py')
+        assert_isdir(self.tmpdir / 'site-packages' / 'module1-0.3.dist-info')
+        self._assert_direct_url(
+            core_samples_dir / 'pep621_nodynamic', 'module1', '0.3',
+            expected_editable=True
+        )
+
     def test_pth_package(self):
         Installer.from_ini_path(samples_dir / 'package1' / 'pyproject.toml', pth=True).install()
         assert_isfile(self.tmpdir / 'site-packages' / 'package1.pth')
@@ -156,6 +233,15 @@ class InstallTests(TestCase):
         assert len(calls) == 1
         assert calls[0]['argv'][1:5] == ['-m', 'pip', 'install', '-r']
 
+    def test_install_reqs_my_python_if_needed_pep621(self):
+        ins = Installer.from_ini_path(
+            core_samples_dir / 'pep621_nodynamic' / 'pyproject.toml',
+            deps='none',
+        )
+
+        # This shouldn't try to get version & docstring from the module
+        ins.install_reqs_my_python_if_needed()
+
     def test_extras_error(self):
         with pytest.raises(DependencyError):
             Installer.from_ini_path(samples_dir / 'requires-requests.toml',
